@@ -12,4 +12,6 @@ from wass2s.was_merge_predictand import *
 from wass2s.was_mme import *
 from wass2s.was_pcr import *
 from wass2s.utils import *
-from wass2s.was_cross_validate import *
\ No newline at end of file
+from wass2s.was_cross_validate import *
+from wass2s.was_transformdata import *
+from wass2s.was_bias_correction import *
\ No newline at end of file
