@@ -10,6 +10,8 @@ import matplotlib.cm as cm
 from matplotlib.patches import Rectangle
 from matplotlib import gridspec
 from matplotlib.offsetbox import  (OffsetImage, AnnotationBbox)
+from mpl_toolkits.axes_grid1.inset_locator import inset_axes
+import matplotlib.image as mpimg
 import matplotlib.image as image
 import cartopy.crs as ccrs
 import cartopy.feature as cfeature
@@ -26,6 +28,7 @@ import matplotlib.patches as mpatches
 from datetime import timedelta
 import cdsapi
 import urllib3
+from scipy.ndimage import gaussian_filter
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
 
@@ -711,7 +714,7 @@ def retrieve_several_zones_for_PCR(dir_to_data, indices_dict, variables_list, ye
     data_vars = [ds.rename(key) for key, ds in predictor.items()]
     return data_vars
 
-def retrieve_single_zone_for_PCR(dir_to_data, indices_dict, variables_list, year_start, year_end, season=None, clim_year_start=None, clim_year_end=None, model=False, month_of_initialization=None, lead_time=None, year_forecast=None):
+def retrieve_single_zone_for_PCR(dir_to_data, indices_dict, variables_list, year_start, year_end, season=None, clim_year_start=None, clim_year_end=None, model=False, month_of_initialization=None, lead_time=None, year_forecast=None, standardize=True):
     """
     Retrieve data for a single zone for Principal Component Regression (PCR) with interpolation.
 
@@ -753,9 +756,11 @@ def retrieve_single_zone_for_PCR(dir_to_data, indices_dict, variables_list, year
         season_str = "".join([calendar.month_abbr[(int(i) + int(month_of_initialization)) % 12 or 12] for i in lead_time])
         center = center.lower().replace("_", "")
         filepath_hdcst = f"{dir_to_data}/hindcast_{center}_{variable}_{abb_month_ini}Ic_{season_str}_{lead_time[0]}.nc"
+        ### A revoir
         filepath_fcst = f"{dir_to_data}/forecast_{center}_{variable}_{abb_month_ini}Ic_{season_str}_{lead_time[0]}.nc"
         data_hdcst = xr.open_dataset(filepath_hdcst).to_array().drop_vars('variable').squeeze('variable')
         data_hdcst['T'] = data_hdcst['T'].astype('datetime64[ns]')
+        ### A revoir
         data_fcst = xr.open_dataset(filepath_fcst).to_array().drop_vars('variable').squeeze('variable')
         data_fcst['T'] = data_fcst['T'].astype('datetime64[ns]')
         data = xr.concat([data_hdcst, data_fcst], dim='T')
@@ -764,11 +769,17 @@ def retrieve_single_zone_for_PCR(dir_to_data, indices_dict, variables_list, year
         filepath = f'{dir_to_data}/{center}_{variable}_{year_start}_{year_end}_{season_str}.nc'
         data = xr.open_dataset(filepath).to_array().drop_vars('variable').squeeze('variable')
         data['T'] = data['T'].astype('datetime64[ns]')
+
     new_resolution = {'Y': 1, 'X': 1}
+
+    if standardize:
+        data = standardize_timeseries(data, clim_year_start, clim_year_end)
+    else:
+        data = anomalize_timeseries(data, clim_year_start, clim_year_end)
+
     for idx, coords in indices_dict.items():
         _, lon_min, lon_max, lat_min, lat_max = coords
         var_region = data.sel(X=slice(lon_min, lon_max), Y=slice(lat_min, lat_max))
-        var_region = standardize_timeseries(var_region, clim_year_start, clim_year_end)
         Y_new = xr.DataArray(np.arange(lat_min, lat_max + 1, new_resolution['Y']), dims='Y')
         X_new = xr.DataArray(np.arange(lon_min, lon_max + 1, new_resolution['X']), dims='X')
         data_vars = var_region.interp(Y=Y_new, X=X_new, method='linear')
@@ -911,10 +922,223 @@ def get_best_models(center_variable, scores, metric='MAE', threshold=None, top_n
             selected_vars_in_order.extend(matches)        
     return selected_vars_in_order # selected_vars
 
+def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-Normal", "Near-Normal", "Above-Normal"], reverse_cmap=True, hspace=-0.6, logo=None, logo_position="lower left", sigma=None, res=None):
+    """
+    Plot probabilistic forecasts with tercile categories.
+
+    Parameters
+    ----------
+    dir_to_save : str
+        Directory path to save the plot.
+    forecast_prob : xarray.DataArray
+        Data array containing probability forecasts with a 'probability' dimension.
+    model_name : str or numpy.ndarray
+        Name of the model for the plot title.
+    labels : list, optional
+        Labels for the tercile categories (default is ["Below-Normal", "Near-Normal", "Above-Normal"]).
+    reverse_cmap : bool, optional
+        If True, reverse the colormap order for categories (default is True).
+    hspace : float, optional
+        Height space between the main plot and the colorbar (default is -0.6).
+    sigma : float, optional
+        Standard deviation for Gaussian smoothing of probabilities (default is None, no smoothing).
+    res : float, optional
+        Resolution for interpolation of the forecast probabilities (default is None, no interpolation). 
+    logo : str, optional
+        Path to the logo image to be added to the plot (default is None).
+    logo_size : float, optional
+        Size of the logo image in the plot (default is 0.5).    
+
+    Notes
+    -----
+    Saves the plot as a PNG file and displays it.
+    Uses custom colormaps for each tercile category.
+    """
+
+    if res is not None:
+        min_X = forecast_prob['X'].min().values
+        max_X = forecast_prob['X'].max().values
+        min_Y = forecast_prob['Y'].min().values
+        max_Y = forecast_prob['Y'].max().values
+        num_X = int((max_X - min_X) / res) + 1
+        num_Y = int((max_Y - min_Y) / res) + 1
+        new_X = np.linspace(min_X, max_X, num_X)
+        new_Y = np.linspace(min_Y, max_Y, num_Y)
+        forecast_prob = forecast_prob.interp(X=new_X, Y=new_Y, method='linear',
+                                                kwargs={'fill_value': 'extrapolate'}    
+                                            )
+
+    if sigma is not None:
+        # Create a smoothed copy
+        forecast_prob_smoothed = forecast_prob * 0.0  # Initialize with same shape and coords
+        
+        # Smooth each probability layer spatially
+        for p in forecast_prob.probability.values:
+            layer = forecast_prob.sel(probability=p)
+            layer_smoothed = xr.apply_ufunc(
+                gaussian_filter,
+                layer,
+                input_core_dims=[['Y', 'X']],
+                output_core_dims=[['Y', 'X']],
+                kwargs={'sigma': sigma}
+            )
+            forecast_prob_smoothed.loc[{'probability': p}] = layer_smoothed
+        
+        # Normalize smoothed probabilities to sum to 1 at each grid point
+        sum_probs = forecast_prob_smoothed.sum('probability')
+        forecast_prob_smoothed = forecast_prob_smoothed / sum_probs.where(sum_probs != 0, 1.0)
+        
+        # Replace original with smoothed
+        forecast_prob = forecast_prob_smoothed
+
+    # Step 1: Extract maximum probability and category
+    max_prob = forecast_prob.max(dim="probability", skipna=True)  # Maximum probability at each grid point
+    # Fill NaN values with a very low value 
+    filled_prob = forecast_prob.fillna(-9999)
+    # Compute argmax
+    max_category = filled_prob.argmax(dim="probability")
+    
+    # Step 2: Create masks for each category
+    mask_bn = max_category == 0  # Below Normal (BN)
+    mask_nn = max_category == 1  # Near Normal (NN)
+    mask_an = max_category == 2  # Above Normal (AN)
+    
+    # Step 3: Define custom colormaps
+    if reverse_cmap:
+        AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ["#ffffe5", "#fff7bc", "#fee391", "#fec44f", "#fe9929", "#ec7014", "#cc4c02", "#993404", "#662506"]) 
+        NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ["#f0f0f0","#d9d9d9", "#bdbdbd", "#969696", "#737373", "#525252"])
+        BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ["#f7fcfd", "#e5f5f9", "#ccece6", "#99d8c9", "#66c2a4", "#41ae76", "#238b45", "#006d2c", "#00441b"])  
+    else:
+        BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ["#ffffe5", "#fff7bc", "#fee391", "#fec44f", "#fe9929", "#ec7014", "#cc4c02", "#993404", "#662506"]) 
+        NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ["#f0f0f0","#d9d9d9", "#bdbdbd", "#969696", "#737373", "#525252"])
+        AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ["#f7fcfd", "#e5f5f9", "#ccece6", "#99d8c9", "#66c2a4", "#41ae76", "#238b45", "#006d2c", "#00441b"])          
+    
+    # Create a figure with GridSpec
+    fig = plt.figure(figsize=(10, 8))
+    gs = gridspec.GridSpec(2, 3, height_ratios=[10, 0.2], width_ratios=[1.2, 0.6, 1.2], hspace=-0.6, wspace=0.2)
+
+    # Main map axis
+    ax = fig.add_subplot(gs[0, :], projection=ccrs.PlateCarree())
+
+    gl = ax.gridlines(draw_labels=True, linewidth=0.05, color='gray', alpha=0.8)
+    gl.top_labels = False
+    gl.right_labels = False
+    
+    bn_data = (max_prob.where(mask_bn) * 100).values
+    nn_data = (max_prob.where(mask_nn) * 100).values
+    an_data = (max_prob.where(mask_an) * 100).values
+
+    # Step 4: Plot each category
+   # def clip_prob(data, mask):
+   #     prob = max_prob.where(mask)
+   #     prob = xr.where(prob > 0.6, 0.6, prob) * 100
+   #     prob = xr.where(prob < 25, 25, prob)
+   #     return prob.values
+
+   # bn_data = clip_prob(max_prob, mask_bn)
+   # nn_data = clip_prob(max_prob, mask_nn)
+   # an_data = clip_prob(max_prob, mask_an)
 
 
+    # Step _: Add  Land colors
+    ax.add_feature(cfeature.LAND.with_scale("50m"), facecolor="#fde0dd", edgecolor="black", zorder=0) # #dfc27d
+    
+    # # Define the data ranges for color normalization  
+    # vmin = 25  # Minimum probability percentage
+    # vmax = 65  # Maximum probability percentage
 
-def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-Normal", "Near-Normal", "Above-Normal"], reverse_cmap=True, logo=None, logo_size=0.5):
+    # Plot BN (Below Normal)
+    if np.any(~np.isnan(bn_data)):
+        bn_plot = ax.pcolormesh(
+            forecast_prob['X'], forecast_prob['Y'], bn_data,
+            cmap=BN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=35, vmax=85
+        )
+    else:
+        bn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=35, vmax=85), cmap=BN_cmap)
+        bn_plot.set_array([])
+
+    # Plot NN (Near Normal)
+    if np.any(~np.isnan(nn_data)):
+        nn_plot = ax.pcolormesh(
+            forecast_prob['X'], forecast_prob['Y'], nn_data,
+            cmap=NN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=35, vmax=65
+        )
+    else:
+        nn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=35, vmax=65), cmap=NN_cmap)
+        nn_plot.set_array([])
+
+    # Plot AN (Above Normal)
+    if np.any(~np.isnan(an_data)):
+        an_plot = ax.pcolormesh(
+            forecast_prob['X'], forecast_prob['Y'], an_data,
+            cmap=AN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=35, vmax=85
+        )
+    else:
+        an_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=35, vmax=85), cmap=AN_cmap)
+        an_plot.set_array([])
+
+    # Step 5: Add coastlines and borders and Land
+    ax.coastlines()
+    ax.add_feature(cfeature.BORDERS, edgecolor='black', linewidth=1.0, linestyle='solid')
+    ax.add_feature(cfeature.OCEAN, facecolor="lightblue")
+    
+    # Step 6: Add individual colorbars with fixed ticks
+    def create_ticks(vn=35, vx=86, step=5):
+        ticks = np.arange(vn, vx, step)
+        return ticks
+
+    #ticks = create_ticks(vn=35, vx=86, step=5)
+
+    # For BN (Below Normal)
+    ticks = create_ticks(vn=35, vx=86, step=5)
+    cbar_ax_bn = fig.add_subplot(gs[1, 0])
+    cbar_bn = plt.colorbar(bn_plot, cax=cbar_ax_bn, orientation='horizontal')
+    cbar_bn.set_label(f'{labels[0]} (%)')
+    cbar_bn.set_ticks(ticks)
+    cbar_bn.set_ticklabels([f"{tick}" for tick in ticks])
+
+    # For NN (Near Normal)
+    ticks = create_ticks(vn=35, vx=66, step=5)
+    cbar_ax_nn = fig.add_subplot(gs[1, 1])
+    cbar_nn = plt.colorbar(nn_plot, cax=cbar_ax_nn, orientation='horizontal')
+    cbar_nn.set_label(f'{labels[1]} (%)')
+    cbar_nn.set_ticks(ticks)
+    cbar_nn.set_ticklabels([f"{tick}" for tick in ticks])
+
+    # For AN (Above Normal)
+    ticks = create_ticks(vn=35, vx=86, step=5)
+    cbar_ax_an = fig.add_subplot(gs[1, 2])
+    cbar_an = plt.colorbar(an_plot, cax=cbar_ax_an, orientation='horizontal')
+    cbar_an.set_label(f'{labels[2]} (%)')
+    cbar_an.set_ticks(ticks)
+    cbar_an.set_ticklabels([f"{tick}" for tick in ticks])
+    
+    # Set the title with the formatted model_name
+    if isinstance(model_name, np.ndarray):
+        model_name_str = str(model_name.item())
+    else:
+        model_name_str = str(model_name)
+    ax.set_title(f"{model_name_str}", fontsize=13, pad=20)
+
+
+    # Step 7: Add logo if provided
+    
+    if logo is not None:
+        ax_logo = inset_axes(ax,
+                            width="7%",          
+                            height="21%",         
+                            loc=logo_position,
+                            borderpad=0.1)        
+        ax_logo.imshow(mpimg.imread(logo))
+        ax_logo.axis("off") 
+
+    # plt.subplots_adjust(top=0.92, bottom=0.08, left=0.08, right=0.92, hspace=0.03, wspace=0.03)
+    plt.subplots_adjust(top=0.95, bottom=0.08, left=0.06, right=0.94, hspace=-0.6, wspace=0.2)
+    plt.savefig(f"{dir_to_save}", dpi=300, bbox_inches='tight')
+    plt.show()
+
+
+def plot_prob_forecasts1(dir_to_save, forecast_prob, model_name, labels=["Below-Normal", "Near-Normal", "Above-Normal"], reverse_cmap=True, logo=None, logo_size=0.5):
     """
     Plot probabilistic forecasts with tercile categories.
 
@@ -953,12 +1177,7 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
     mask_an = max_category == 2  # Above Normal (AN)
     
     # Step 3: Define custom colormaps
-    # BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ['#FFF5F0', '#FB6A4A', '#67000D'])
-    # NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ['#F7FCF5', '#74C476', '#00441B'])
-    # AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#F7FBFF', '#6BAED6', '#08306B'])
-
     if reverse_cmap:
-        
         AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#FDAE61', '#F46D43', '#D73027']) 
         NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ['#FFFFE5', '#FFF7BC', '#FEE391'])
         BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ['#ABDDA4', '#66C2A5', '#3288BD'])  
@@ -968,12 +1187,20 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
         AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#ABDDA4', '#66C2A5', '#3288BD'])          
     
     # Create a figure with GridSpec
-    fig = plt.figure(figsize=(8, 6))
-    gs = gridspec.GridSpec(2, 3, height_ratios=[15, 0.5])
-    
+    # fig = plt.figure(figsize=(8, 6.5))  # Increased height to accommodate logo
+    # gs = gridspec.GridSpec(3, 3, height_ratios=[15, 0.8, 0.7], hspace=0.2)
+
+    fig = plt.figure(figsize=(10, 8))
+    gs = gridspec.GridSpec(3, 3, height_ratios=[8, 0.2, 3], hspace=0.03)
+
     # Main map axis
     ax = fig.add_subplot(gs[0, :], projection=ccrs.PlateCarree())
+
+    # Modify by Mandela
     
+    ###################
+    ##################
+
     # Step 4: Plot each category
     # Multiply by 100 to convert probabilities to percentages
     
@@ -991,20 +1218,17 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
     #                    xr.where(max_prob.where(mask_nn) * 100 > 60, 60, max_prob.where(mask_nn) * 100)).values
     # an_data = xr.where((max_prob.where(mask_an) * 100) < 45, 45,
     #                    xr.where(max_prob.where(mask_an) * 100 > 60, 60, max_prob.where(mask_an) * 100)).values    
-    
+
+
+    # Step 4: Plot each category
     bn_data = xr.where((xr.where(max_prob.where(mask_bn)>0.6,0.6,max_prob.where(mask_bn))* 100)<45, 45,
                        xr.where(max_prob.where(mask_bn)>0.6,0.6,max_prob.where(mask_bn))* 100).values  
     nn_data = xr.where((xr.where(max_prob.where(mask_nn)>0.6,0.6,max_prob.where(mask_nn))* 100)<45, 45,
                    xr.where(max_prob.where(mask_nn)>0.6,0.6,max_prob.where(mask_nn))* 100).values
     an_data = xr.where((xr.where(max_prob.where(mask_an)>0.6,0.6,max_prob.where(mask_an))* 100)<45, 45,
                    xr.where(max_prob.where(mask_an)>0.6,0.6,max_prob.where(mask_an))* 100).values
-     
-
     
     # Define the data ranges for color normalization  
-    # vmin and vmax are set to 35% and 65% respectively for all categories
-    # This ensures that the color mapping is consistent across all plots 
-     
     vmin = 35  # Minimum probability percentage
     vmax = 65  # Maximum probability percentage
 
@@ -1015,7 +1239,6 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
             cmap=BN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
-        # Create a dummy mappable for BN
         bn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=BN_cmap)
         bn_plot.set_array([])
 
@@ -1026,7 +1249,6 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
             cmap=NN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
-        # Create a dummy mappable for NN
         nn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=NN_cmap)
         nn_plot.set_array([])
 
@@ -1037,7 +1259,6 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
             cmap=AN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
-        # Create a dummy mappable for AN
         an_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=AN_cmap)
         an_plot.set_array([])
 
@@ -1046,8 +1267,6 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
     ax.add_feature(cfeature.BORDERS, linestyle=':')
     
     # Step 6: Add individual colorbars with fixed ticks
-    
-    # Function to create ticks at intervals of 10 from 0 to 100
     def create_ticks():
         ticks = np.arange(35, 66, 5)
         return ticks
@@ -1076,30 +1295,28 @@ def plot_prob_forecasts(dir_to_save, forecast_prob, model_name, labels=["Below-N
     cbar_an.set_ticklabels([f"{tick}" for tick in ticks])
     
     # Set the title with the formatted model_name
-    # Convert model_name to string if necessary
     if isinstance(model_name, np.ndarray):
         model_name_str = str(model_name.item())
     else:
         model_name_str = str(model_name)
     ax.set_title(f"{model_name_str}", fontsize=13, pad=20)
-    # Step 7: Add logo if provided
-    if logo_path is not None:
-        im = image.imread(logo)
-        # add logo
-        addLogo = OffsetImage(im, zoom=logo_size)
-        # addLogo.image.axes = ax
-        addLogo.set_offset('lower right')
-        ab = AnnotationBbox(addLogo, (1.1, -0.1), frameon=False, xycoords='axes fraction', boxcoords="axes fraction")
-        ax.add_artist(ab)   
-    # Step 8: Adjust layout and save the plot
-    plt.tight_layout()
-    plt.savefig(f"{dir_to_save}/{model_name_str.replace(" ", "_")}.png", dpi=300, bbox_inches='tight')
-    plt.show()
-    
 
-def plot_prob_forecasts_(dir_to_save, forecast_prob, model_name):
+    # Step 7: Add logo if provided
+    logo_ax = fig.add_subplot(gs[2, 2])
+    logo_ax.axis('off')
+    if logo is not None:
+        im = image.imread(logo)
+        addLogo = OffsetImage(im, zoom=logo_size)
+        ab = AnnotationBbox(addLogo, (0.5, 0.5), frameon=False, xycoords='axes fraction')
+        logo_ax.add_artist(ab)
+
+    plt.subplots_adjust(top=0.92, bottom=0.08, left=0.08, right=0.92, hspace=-0.6, wspace=0.2)
+    plt.savefig(f"{dir_to_save}/{model_name_str.replace(' ', '_')}.png", dpi=300, bbox_inches='tight')
+    plt.show()
+
+def plot_prob_forecasts2(dir_to_save, forecast_prob, model_name, labels=["Below-Normal", "Near-Normal", "Above-Normal"], reverse_cmap=True, logo=None, logo_size=0.5):
     """
-    Plot probabilistic forecasts with tercile categories using contourf.
+    Plot probabilistic forecasts with tercile categories.
 
     Parameters
     ----------
@@ -1109,79 +1326,172 @@ def plot_prob_forecasts_(dir_to_save, forecast_prob, model_name):
         Data array containing probability forecasts with a 'probability' dimension.
     model_name : str or numpy.ndarray
         Name of the model for the plot title.
+    labels : list, optional
+        Labels for the tercile categories (default is ["Below-Normal", "Near-Normal", "Above-Normal"]).
+    reverse_cmap : bool, optional
+        If True, reverse the colormap order for categories (default is True).
+    logo : str, optional
+        Path to the logo image to be added to the plot (default is None).
+    logo_size : float, optional
+        Size of the logo image in the plot (default is 0.5).    
 
     Notes
     -----
-    Similar to plot_prob_forecasts but uses contourf instead of pcolormesh for plotting.
     Saves the plot as a PNG file and displays it.
+    Uses custom colormaps for each tercile category.
     """
-    max_prob = forecast_prob.max(dim="probability", skipna=True)
+    # Step 1: Extract maximum probability and category
+    max_prob = forecast_prob.max(dim="probability", skipna=True)  # Maximum probability at each grid point
+    # Fill NaN values with a very low value 
     filled_prob = forecast_prob.fillna(-9999)
+    # Compute argmax
     max_category = filled_prob.argmax(dim="probability")
-    mask_bn = max_category == 0
-    mask_nn = max_category == 1
-    mask_an = max_category == 2
-    BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ['#FDAE61', '#F46D43', '#D73027'])
-    NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ['#FFFFE5', '#FFF7BC', '#FFFFCC'])
-    AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#ABDDA4', '#66C2A5', '#3288BD'])
-    fig = plt.figure(figsize=(8, 6))
-    gs = gridspec.GridSpec(2, 3, height_ratios=[15, 0.5])
+    
+    # Step 2: Create masks for each category
+    mask_bn = max_category == 0  # Below Normal (BN)
+    mask_nn = max_category == 1  # Near Normal (NN)
+    mask_an = max_category == 2  # Above Normal (AN)
+    
+    # Step 3: Define custom colormaps
+    if reverse_cmap:
+        AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#FDAE61', '#F46D43', '#D73027']) 
+        NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ['#FFFFE5', '#FFF7BC', '#FEE391'])
+        BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ['#ABDDA4', '#66C2A5', '#3288BD'])  
+    else:
+        BN_cmap = mcolors.LinearSegmentedColormap.from_list('BN', ['#FDAE61', '#F46D43', '#D73027']) 
+        NN_cmap = mcolors.LinearSegmentedColormap.from_list('NN', ['#FFFFE5', '#FFF7BC', '#FEE391'])
+        AN_cmap = mcolors.LinearSegmentedColormap.from_list('AN', ['#ABDDA4', '#66C2A5', '#3288BD'])          
+    
+    # Create a figure with GridSpec
+    # fig = plt.figure(figsize=(8, 6.5))  # Increased height to accommodate logo
+    # gs = gridspec.GridSpec(3, 3, height_ratios=[15, 0.8, 0.7], hspace=0.2)
+
+    fig = plt.figure(figsize=(10, 8))
+    gs = gridspec.GridSpec(3, 3, height_ratios=[8, 0.2, 3], hspace=0.03)
+
+    # Main map axis
     ax = fig.add_subplot(gs[0, :], projection=ccrs.PlateCarree())
-    bn_data = xr.where((xr.where(max_prob.where(mask_bn) > 0.6, 0.6, max_prob.where(mask_bn)) * 100) < 45, 45,
-                       xr.where(max_prob.where(mask_bn) > 0.6, 0.6, max_prob.where(mask_bn)) * 100).values
-    nn_data = xr.where((xr.where(max_prob.where(mask_nn) > 0.6, 0.6, max_prob.where(mask_nn)) * 100) < 45, 45,
-                       xr.where(max_prob.where(mask_nn) > 0.6, 0.6, max_prob.where(mask_nn)) * 100).values
-    an_data = xr.where((xr.where(max_prob.where(mask_an) > 0.6, 0.6, max_prob.where(mask_an)) * 100) < 45, 45,
-                       xr.where(max_prob.where(mask_an) > 0.6, 0.6, max_prob.where(mask_an)) * 100).values
-    vmin, vmax = 35, 65
+
+    # Modify by Mandela
+    
+    ###################
+    ##################
+    sigma = 2.0  # Smoothing parameter (Gaussian sigma); 
+    
+    # Create a smoothed copy
+    forecast_prob_smoothed = forecast_prob * 0.0  # Initialize with same shape and coords
+    
+    # Smooth each probability layer spatially
+    for p in forecast_prob.probability.values:
+        layer = forecast_prob.sel(probability=p)
+        layer_smoothed = xr.apply_ufunc(
+            gaussian_filter,
+            layer,
+            input_core_dims=[['Y', 'X']],
+            output_core_dims=[['Y', 'X']],
+            kwargs={'sigma': sigma}
+        )
+        forecast_prob_smoothed.loc[{'probability': p}] = layer_smoothed
+    
+    # Normalize smoothed probabilities to sum to 1 at each grid point
+    sum_probs = forecast_prob_smoothed.sum('probability')
+    forecast_prob_smoothed = forecast_prob_smoothed / sum_probs.where(sum_probs != 0, 1.0)
+    
+    # Replace original with smoothed
+    forecast_prob = forecast_prob_smoothed
+
+    # Step 4: Plot each category
+    bn_data = xr.where((xr.where(max_prob.where(mask_bn)>0.6,0.6,max_prob.where(mask_bn))* 100)<45, 45,
+                       xr.where(max_prob.where(mask_bn)>0.6,0.6,max_prob.where(mask_bn))* 100).values  
+    nn_data = xr.where((xr.where(max_prob.where(mask_nn)>0.6,0.6,max_prob.where(mask_nn))* 100)<45, 45,
+                   xr.where(max_prob.where(mask_nn)>0.6,0.6,max_prob.where(mask_nn))* 100).values
+    an_data = xr.where((xr.where(max_prob.where(mask_an)>0.6,0.6,max_prob.where(mask_an))* 100)<45, 45,
+                   xr.where(max_prob.where(mask_an)>0.6,0.6,max_prob.where(mask_an))* 100).values
+    
+    # Define the data ranges for color normalization  
+    vmin = 35  # Minimum probability percentage
+    vmax = 65  # Maximum probability percentage
+
+    # Plot BN (Below Normal)
     if np.any(~np.isnan(bn_data)):
-        bn_plot = ax.contourf(
+        bn_plot = ax.pcolormesh(
             forecast_prob['X'], forecast_prob['Y'], bn_data,
             cmap=BN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
         bn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=BN_cmap)
         bn_plot.set_array([])
+
+    # Plot NN (Near Normal)
     if np.any(~np.isnan(nn_data)):
-        nn_plot = ax.contourf(
+        nn_plot = ax.pcolormesh(
             forecast_prob['X'], forecast_prob['Y'], nn_data,
             cmap=NN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
         nn_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=NN_cmap)
         nn_plot.set_array([])
+
+    # Plot AN (Above Normal)
     if np.any(~np.isnan(an_data)):
-        an_plot = ax.contourf(
+        an_plot = ax.pcolormesh(
             forecast_prob['X'], forecast_prob['Y'], an_data,
             cmap=AN_cmap, transform=ccrs.PlateCarree(), alpha=0.9, vmin=vmin, vmax=vmax
         )
     else:
         an_plot = cm.ScalarMappable(norm=plt.Normalize(vmin=vmin, vmax=vmax), cmap=AN_cmap)
         an_plot.set_array([])
+
+    # Step 5: Add coastlines and borders
     ax.coastlines()
     ax.add_feature(cfeature.BORDERS, linestyle=':')
+    
+    # Step 6: Add individual colorbars with fixed ticks
     def create_ticks():
-        return np.arange(35, 66, 5)
+        ticks = np.arange(35, 66, 5)
+        return ticks
+
     ticks = create_ticks()
+
+    # For BN (Below Normal)
     cbar_ax_bn = fig.add_subplot(gs[1, 0])
     cbar_bn = plt.colorbar(bn_plot, cax=cbar_ax_bn, orientation='horizontal')
-    cbar_bn.set_label('Below-Normal (%)')
+    cbar_bn.set_label(f'{labels[0]} (%)')
     cbar_bn.set_ticks(ticks)
     cbar_bn.set_ticklabels([f"{tick}" for tick in ticks])
+
+    # For NN (Near Normal)
     cbar_ax_nn = fig.add_subplot(gs[1, 1])
     cbar_nn = plt.colorbar(nn_plot, cax=cbar_ax_nn, orientation='horizontal')
-    cbar_nn.set_label('Near-Normal (%)')
+    cbar_nn.set_label(f'{labels[1]} (%)')
     cbar_nn.set_ticks(ticks)
     cbar_nn.set_ticklabels([f"{tick}" for tick in ticks])
+
+    # For AN (Above Normal)
     cbar_ax_an = fig.add_subplot(gs[1, 2])
     cbar_an = plt.colorbar(an_plot, cax=cbar_ax_an, orientation='horizontal')
-    cbar_an.set_label('Above-Normal (%)')
+    cbar_an.set_label(f'{labels[2]} (%)')
     cbar_an.set_ticks(ticks)
     cbar_an.set_ticklabels([f"{tick}" for tick in ticks])
-    model_name_str = str(model_name.item()) if isinstance(model_name, np.ndarray) else str(model_name)
-    ax.set_title(f"Forecast Probabilities - {model_name_str}", fontsize=14, pad=20)
-    plt.tight_layout()
-    plt.savefig(f"{dir_to_save}/Forecast_{model_name_str}_.png", dpi=300, bbox_inches='tight')
+    
+    # Set the title with the formatted model_name
+    if isinstance(model_name, np.ndarray):
+        model_name_str = str(model_name.item())
+    else:
+        model_name_str = str(model_name)
+    ax.set_title(f"{model_name_str}", fontsize=13, pad=20)
+
+    # Step 7: Add logo if provided
+    logo_ax = fig.add_subplot(gs[2, 2])
+    logo_ax.axis('off')
+    if logo is not None:
+        im = image.imread(logo)
+        addLogo = OffsetImage(im, zoom=logo_size)
+        ab = AnnotationBbox(addLogo, (0.5, 0.5), frameon=False, xycoords='axes fraction')
+        logo_ax.add_artist(ab)
+
+    plt.subplots_adjust(top=0.92, bottom=0.08, left=0.08, right=0.92, hspace=0.03, wspace=0.3)
+    plt.savefig(f"{dir_to_save}/{model_name_str.replace(' ', '_')}.png", dpi=300, bbox_inches='tight')
     plt.show()
 
 def plot_tercile(A):
@@ -1310,8 +1620,7 @@ def process_model_for_other_params(agmParamModel, dir_to_save, hdcst_file_path,
     x_coords = obs_fcst_year.X
 
     if calendar.isleap(year_forecast):
-        dayofyear = obs_hdcst['T'].dt.dayofyear
-        daily_climatology = da.groupby(dayofyear).mean(dim='T')
+        daily_climatology = obs_hdcst.groupby('T.dayofyear').mean(dim='T')
         data = daily_climatology.to_numpy()
         dummy = xr.DataArray(
             data=data,
@@ -1319,8 +1628,10 @@ def process_model_for_other_params(agmParamModel, dir_to_save, hdcst_file_path,
             dims=["T", "Y", "X"]
         ) * mask
     else:
-        da_noleap = obs_hdcst.sel(T=~((obs_hdcst['T'].dt.month == 2) & (obs_hdcst['T'].dt.day == 29)))
-        daily_climatology = da_noleap.groupby('T.dayofyear').mean(dim='T')
+        # da_noleap = obs_hdcst.sel(T=~((obs_hdcst['T'].dt.month == 2) & (obs_hdcst['T'].dt.day == 29)))
+        # daily_climatology = da_noleap.groupby('T.dayofyear').mean(dim='T')
+        daily_climatology = obs_hdcst.groupby('T.dayofyear').mean(dim='T')
+        daily_climatology = daily_climatology.sel(dayofyear=~((daily_climatology['dayofyear'] == 60)))
         data = daily_climatology.to_numpy()
         dummy = xr.DataArray(
             data=data,
@@ -1661,7 +1972,10 @@ fcst_file_path, obs_hdcst, obs_fcst_year, month_of_initialization,
             filled_ds = ds1_aligned.fillna(ds2_aligned)
             ds_filled = filled_ds.copy()
             ds_filled = ds_filled.sortby("T")
+            if param=="PRCP":
+                ds_filled.where(ds_filled >= 0, 0)
             ds_processed = ds_filled.to_dataset(name=param)
+            ds_processed = ds_processed.transpose("T", "Y", "X")
             ds_processed.to_netcdf(save_path)
         else:
             print(f"[SKIP] {save_path} already exists.")
@@ -1685,7 +1999,10 @@ fcst_file_path, obs_hdcst, obs_fcst_year, month_of_initialization,
             filled_fcst_ = ds1_aligned.fillna(ds2_aligned)
             ds_filled = filled_fcst_.copy()
             ds_filled = ds_filled.sortby("T")
+            if param=="PRCP":
+                ds_filled.where(ds_filled >= 0, 0)
             ds_processed = ds_filled.to_dataset(name=param)
+            ds_processed = ds_processed.transpose("T", "Y", "X")
             ds_processed.to_netcdf(save_path)
         else:
             print(f"[SKIP] {save_path} already exists.")
@@ -1770,7 +2087,7 @@ def svp(T):
     """
     return 0.6108 * np.exp(17.27 * T / (T + 237.3))
 
-def et0_fao56_daily(tmax, tmin, tdew, u10, v10, rs, mlsp, dem):
+def et0_fao56_daily(tmax, tmin, rs, mlsp, dem, tdew=None, u10=None, v10=None, wff=None):
     """
     Compute daily reference evapotranspiration (ET₀) using the FAO-56 Penman-Monteith equation.
 
@@ -1785,14 +2102,16 @@ def et0_fao56_daily(tmax, tmin, tdew, u10, v10, rs, mlsp, dem):
         Daily minimum temperature in °C, with dimensions ('T', 'Y', 'X').
     tdew : xarray.DataArray
         Daily mean dew point temperature in °C, with dimensions ('T', 'Y', 'X').
-    u10 : xarray.DataArray
+    u10 : xarray.DataArray, optional
         Daily mean zonal wind speed at 10 m height in m s^-1, with dimensions ('T', 'Y', 'X').
-    v10 : xarray.DataArray
+    v10 : xarray.DataArray, optional
         Daily mean meridional wind speed at 10 m height in m s^-1, with dimensions ('T', 'Y', 'X').
+    wff : xarray.DataArray, optional
+        Daily mean wind speed at 10 m height in m s^-1, with dimensions ('T', 'Y', 'X').
     rs : xarray.DataArray
         Daily mean incoming solar radiation in W m^-2 or MJ m^-2 day^-1, with dimensions ('T', 'Y', 'X').
     mlsp : xarray.DataArray
-        Daily mean sea-level pressure in Pa, with dimensions ('T', 'Y', 'X').
+        Daily mean sea-level pressure in hPa, with dimensions ('T', 'Y', 'X').
     dem : xarray.DataArray
         Digital elevation model (terrain height) in meters, with dimensions ('Y', 'X').
 
@@ -1834,16 +2153,28 @@ def et0_fao56_daily(tmax, tmin, tdew, u10, v10, rs, mlsp, dem):
     Δ = 4098 * svp(tmean) / (tmean + 237.3)**2  # Slope of svp curve (kPa °C^-1)
 
     # --- 3 Pressures ---
-    P0 = mlsp / 1000.0  # Pa to kPa
+    P0 = mlsp / 10.0  # hPa to kPa
     P = P0 * (1 - 0.0065 * dem / (tmean + 273.15))**5.257  # Hypsometric equation
     PSI = 0.665e-3 * P  # Psychrometric constant (kPa °C^-1)
 
     # --- 4 Wind ---
-    ws10 = np.hypot(u10, v10)  # Wind speed at 10 m
-    u2 = ws10 * 4.87 / np.log(67.8 * 10.0 - 5.42)  # Wind speed at 2 m
+    if wff is not None:
+        ws10 = wff
+        u2 = ws10 * 4.87 / np.log(67.8 * 10.0 - 5.42)  # Wind speed at 2 m
+    elif u10 is not None and v10 is not None: # Use u10 and v10 if available
+        ws10 = np.hypot(u10, v10)  # Wind speed at 10 m
+        u2 = ws10 * 4.87 / np.log(67.8 * 10.0 - 5.42)  # Wind speed at 2 m
+    else:
+        raise ValueError("Either 'wff' or both 'u10' and 'v10' must be provided for wind speed calculation.")
+
 
     # --- 5 Vapour pressures ---
-    ea = svp(tdew)  # Actual vapor pressure (kPa)
+    if tdew is None:
+        # If dew point is not provided, use tmin
+        ea = svp(tmin)  # Actual vapor pressure (kPa)
+    else:
+        ea = svp(tdew)  # Actual vapor pressure (kPa)
+    
     es = (svp(tmax) + svp(tmin)) / 2  # Saturated vapor pressure (kPa)
     vpd = es - ea  # Vapor pressure deficit (kPa)
 
@@ -1876,7 +2207,6 @@ def et0_fao56_daily(tmax, tmin, tdew, u10, v10, rs, mlsp, dem):
     et0.name = "ET0"
     return et0
 
-
 # Other commmented code to use after
 
 # retrieve Zone for PCR
@@ -1890,6 +2220,7 @@ def et0_fao56_daily(tmax, tmin, tdew, u10, v10, rs, mlsp, dem):
 # expl
 
 
+
 # def plot_prob_forecats(dir_to_save, forecast_prob, model_name):    
 #     # Step 1: Extract maximum probability and category
 #     max_prob = forecast_prob.max(dim="probability", skipna=True)  # Maximum probability at each grid point
