@@ -166,13 +166,12 @@ class WAS_Cross_Validator:
         hindcast_det = []
         hindcast_prob = []
         n_splits = len(Predictant.get_index("T"))
-        same_prob_method = [WAS_Ridge_Model, WAS_Lasso_Model, WAS_LassoLars_Model, WAS_ElasticNet_Model,
+        same_prob_method = [WAS_Ridge_Model, WAS_Lasso_Model, WAS_LassoLars_Model, WAS_ElasticNet_Model, WAS_MARS_Model,
                             WAS_LinearRegression_Model, WAS_SVR, WAS_PolynomialRegression, WAS_PoissonRegression]
-        same_kind_model1 = [WAS_mme_ELM]
+        # same_kind_model1 = [WAS_mme_xcELM]
         same_kind_model2 = [WAS_mme_MLP, WAS_mme_GradientBoosting, WAS_mme_XGBoosting, WAS_mme_AdaBoost,
-                            WAS_mme_LGBM_Boosting, WAS_mme_Stack_MLP_RF, WAS_mme_Stack_Lasso_RF_MLP,
-                            WAS_mme_Stack_MLP_Ada_Ridge, WAS_mme_Stack_RF_GB_Ridge, WAS_mme_Stack_KNN_Tree_SVR,
-                            WAS_mme_GA]
+                            WAS_mme_Stack_RF_MLP, WAS_mme_hpELM, WAS_mme_RF_hpELM, WAS_mme_StackXGboost_hpELM,
+                            WAS_mme_RF, WAS_mme_StackXGboost_MLP, WAS_mme_GA]
         same_kind_model3 = [WAS_RandomForest_XGBoost_ML_Stacking, WAS_MLP, WAS_Stacking_Ridge,
                             WAS_RandomForest_XGBoost_Stacking_MLP]
 
@@ -199,6 +198,7 @@ class WAS_Cross_Validator:
 
         elif isinstance(model, WAS_Analog):
 
+            # revoir l'option dutiliser download_and_process ici, enfin deviter les repetitions
 
             print("Cross-validation ongoing")
             for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(np.unique(Predictant['T'].dt.year), self.nb_omit), total=n_splits), start=1):
@@ -211,7 +211,7 @@ class WAS_Cross_Validator:
             hindcast_prob = model.compute_prob(Predictant, clim_year_start, clim_year_end, hindcast_det)
             return hindcast_det, hindcast_prob
 
-        elif any(isinstance(model, i) for i in same_kind_model1):
+        elif isinstance(model, WAS_mme_xcELM):
             print("Cross-validation ongoing")
             for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['S'], self.nb_omit), total=n_splits), start=1):
                 X_train, X_test = Predictor.isel(S=train_index), Predictor.isel(S=test_index)
@@ -225,7 +225,7 @@ class WAS_Cross_Validator:
 
             return hindcast_det.load(), hindcast_prob.load()
 
-        elif isinstance(model, WAS_mme_ELR):
+        elif isinstance(model, WAS_mme_xcELR):
             print("Cross-validation ongoing")
             for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['S'], self.nb_omit), total=n_splits), start=1):
                 X_train, X_test = Predictor.isel(S=train_index), Predictor.isel(S=test_index)
@@ -235,7 +235,94 @@ class WAS_Cross_Validator:
 
             hindcast_det = xr.concat(hindcast_det, dim="T")
             hindcast_det['T'] = Predictant['T']
-            return hindcast_det.load()
+            return hindcast_det.load(), hindcast_det.load()
+
+        elif isinstance(model, WAS_mme_RoebberGA):
+
+            Predictor = Predictor.fillna(Predictor.mean(dim="T", skipna=True))
+            Predictant = Predictant.fillna(Predictant.mean(dim="T", skipna=True))
+            
+            if "M" in Predictant.coords:
+                Predictant = Predictant.isel(M=0).drop_vars('M').squeeze()
+            else:
+                Predictant = Predictant
+
+            print("Cross-validation ongoing")
+            for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['T'], self.nb_omit), total=n_splits), start=1):
+                X_train, X_test = Predictor.isel(T=train_index), Predictor.isel(T=test_index)
+                y_train, y_test = Predictant.isel(T=train_index), Predictant.isel(T=test_index)
+                pred_det = model.compute_model(X_train, y_train, X_test)
+                hindcast_det.append(pred_det)
+
+            hindcast_det = xr.concat(hindcast_det, dim="T")
+            hindcast_det['T'] = Predictant['T']
+            hindcast_det = hindcast_det
+            hindcast_prob = model.compute_prob(Predictant, clim_year_start, clim_year_end, hindcast_det)
+
+            return hindcast_det, hindcast_prob
+
+        elif isinstance(model, WAS_mme_NGR_Model):
+
+            Predictor = Predictor.fillna(Predictor.mean(dim="T", skipna=True))
+            Predictant = Predictant.fillna(Predictant.mean(dim="T", skipna=True))
+
+            if "M" in Predictant.coords:
+                Predictant = Predictant.isel(M=0).drop_vars('M').squeeze()
+            else:
+                Predictant = Predictant
+
+            print("Cross-validation ongoing")
+            for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['T'], self.nb_omit), total=n_splits), start=1):
+                X_train, X_test = Predictor.isel(T=train_index), Predictor.isel(T=test_index)
+                y_train, y_test = Predictant.isel(T=train_index), Predictant.isel(T=test_index)
+                pred_det = model.compute_model(X_train, y_train, X_test, Predictant=Predictant, clim_year_start=clim_year_start, clim_year_end=clim_year_end)
+                hindcast_det.append(pred_det)
+
+            hindcast_det = xr.concat(hindcast_det, dim="T")
+            hindcast_det['T'] = Predictant['T']
+            return hindcast_det, hindcast_det
+
+        elif isinstance(model, WAS_mme_FlexibleNGR_Model):
+
+            Predictor = Predictor.fillna(Predictor.mean(dim="T", skipna=True))
+            Predictant = Predictant.fillna(Predictant.mean(dim="T", skipna=True))
+
+            if "M" in Predictant.coords:
+                Predictant = Predictant.isel(M=0).drop_vars('M').squeeze()
+            else:
+                Predictant = Predictant
+
+            print("Cross-validation ongoing")
+            for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['T'], self.nb_omit), total=n_splits), start=1):
+                X_train, X_test = Predictor.isel(T=train_index), Predictor.isel(T=test_index)
+                y_train, y_test = Predictant.isel(T=train_index), Predictant.isel(T=test_index)
+                pred_det = model.compute_model(X_train, y_train, X_test, Predictant=Predictant, clim_year_start=clim_year_start, clim_year_end=clim_year_end)
+                hindcast_det.append(pred_det)
+
+            hindcast_det = xr.concat(hindcast_det, dim="T")
+            hindcast_det['T'] = Predictant['T']
+            return hindcast_det, hindcast_det
+
+        elif isinstance(model, WAS_mme_BMA_Sloughter):
+
+            Predictor = Predictor.fillna(Predictor.mean(dim="T", skipna=True))
+            Predictant = Predictant.fillna(Predictant.mean(dim="T", skipna=True))
+
+            print("Cross-validation ongoing")
+            for i, (train_index, test_index) in enumerate(tqdm(self.custom_cv.split(Predictor['T'], self.nb_omit), total=n_splits), start=1):
+                X_train, X_test = Predictor.isel(T=train_index), Predictor.isel(T=test_index)
+                y_train, y_test = Predictant.isel(T=train_index), Predictant.isel(T=test_index)
+                print(X_train, y_train, X_test)
+                pred_det = model.compute_model(X_train, y_train, X_test)
+                pred_prob = model.compute_prob(X_train, y_train, X_test, Predictant=Predictant, clim_year_start=clim_year_start, clim_year_end=clim_year_end)
+                hindcast_det.append(pred_det)
+                hindcast_prob.append(pred_prob)
+
+            hindcast_det = xr.concat(hindcast_det, dim="T")
+            hindcast_prob = xr.concat(hindcast_prob, dim="T")
+            hindcast_det['T'] = Predictant['T']
+            hindcast_prob['T'] = Predictant['T']
+            return hindcast_det, hindcast_prob
 
         elif any(isinstance(model, i) for i in same_kind_model2):
             mask = xr.where(~np.isnan(Predictant.isel(T=0)), 1, np.nan).drop_vars(['T']).squeeze().to_numpy()
